@@ -1,5 +1,6 @@
 //! Tibco EMS binding.
 
+use enum_extract::extract;
 use log::{error, trace};
 use serde::{Deserialize, Serialize};
 use std::collections::HashMap;
@@ -1286,6 +1287,20 @@ fn build_message_pointer_from_message(message: &Message) -> usize {
       Message::ObjectMessage(msg) => msg.header.clone(),
     };
     if let Some(headers) = header {
+      //look for correlation id
+      if let Some(correlation_id) = headers.get("CorrelationID") {
+        let correlation_id_val = extract!(TypedValue::String(_), correlation_id).expect("extract correlation id");
+        let c_correlation_id = CString::new(correlation_id_val.as_str()).unwrap();
+        let status = tibco_ems_sys::tibemsMsg_SetCorrelationID(
+          msg_pointer,
+          c_correlation_id.as_ptr(),
+        );
+        match status {
+          tibems_status::TIBEMS_OK => trace!("tibemsMsg_SetCorrelationId: {:?}", status),
+          _ => error!("tibemsMsg_SetCorrelationId: {:?}", status),
+        }
+      }
+      //do other headers (also do correlation id again as custom header)
       for (key, val) in &headers {
         let c_name = CString::new(key.to_string()).unwrap();
         match val {
@@ -1504,20 +1519,32 @@ fn build_message_from_pointer(msg_pointer: usize) -> Message {
           "MessageID".to_string(),
           TypedValue::String(message_id.to_string()),
         );
-        //extract body
-        let buf_vec: Vec<i8> = vec![0; 0];
-        let buf_ref: *const std::os::raw::c_char = buf_vec.as_ptr();
-        let mut result_size: u32 = 0;
-        let status =
-          tibco_ems_sys::tibemsBytesMsg_GetBytes(msg_pointer, &buf_ref, &mut result_size);
+        //check body length
+        let mut body_length: i32 = 0;
+        let mut body_value: Vec<u8> = vec![0; 0];
+        let status = tibco_ems_sys::tibemsBytesMsg_GetBodyLength(msg_pointer, &mut body_length);
         match status {
-          tibems_status::TIBEMS_OK => trace!("tibemsBytesMsg_GetBytes: {:?}", status),
-          _ => error!("tibemsBytesMsg_GetBytes: {:?}", status),
+          tibems_status::TIBEMS_OK =>{
+            trace!("tibemsBytesMsg_GetBodyLength: {:?}", status);
+            if body_length > 0 {
+              //extract body
+              let buf_vec: Vec<i8> = vec![0; 0];
+              let buf_ref: *const std::os::raw::c_char = buf_vec.as_ptr();
+              let mut result_size: u32 = 0;
+              let status =
+                tibco_ems_sys::tibemsBytesMsg_GetBytes(msg_pointer, &buf_ref, &mut result_size);
+              match status {
+                tibems_status::TIBEMS_OK => trace!("tibemsBytesMsg_GetBytes: {:?}", status),
+                _ => error!("tibemsBytesMsg_GetBytes: {:?}", status),
+              }
+              body_value = CStr::from_ptr(buf_ref).to_bytes().to_vec();
+              body_value.truncate(result_size as usize);
+            }
+          },
+          _ => error!("tibemsBytesMsg_GetBodyLength: {:?}", status),
         }
-        let mut x = CStr::from_ptr(buf_ref).to_bytes().to_vec();
-        x.truncate(result_size as usize);
         msg = Message::BytesMessage(BytesMessage {
-          body: x,
+          body: body_value,
           header: None,
           pointer: Some(msg_pointer),
           destination: None,
@@ -1568,11 +1595,14 @@ fn build_message_from_pointer(msg_pointer: usize) -> Message {
     match status {
       tibems_status::TIBEMS_OK =>{
         trace!("tibemsMsg_GetCorrelationID: {:?}", status);
-        let correlation_id = CStr::from_ptr(buf_ref).to_str().unwrap();
-        header.insert(
-          "CorrelationID".to_string(),
-          TypedValue::String(correlation_id.to_string()),
-        );    
+        // check for null pointer (when no correlation id was set)
+        if buf_ref != std::ptr::null() {
+          let correlation_id = CStr::from_ptr(buf_ref).to_str().unwrap();
+          header.insert(
+            "CorrelationID".to_string(),
+            TypedValue::String(correlation_id.to_string()),
+          );
+        }
       },
       _ => trace!("tibemsMsg_GetCorrelationID: {:?}", status),
     }
